use super::models;
use beteran_common_rust as bcr;
use diesel::{result::Error, sql_query, RunQueryDsl};
use std::fmt::Write;

static BETTING_HISTORY_COUNT_QUERY: &str = "
SELECT
  COUNT(akbh.id)
FROM api_kgon_betting_history as akbh
  INNER JOIN members m
    ON m.username = akbh.site_username
  LEFT OUTER JOIN member_settlement_settings m_mss
    ON m_mss.member_id = m.id
";

static BETTING_HISTORY_QUERY: &str = "
SELECT
  akbh.id                   as akbh_id            ,
  akbh.vendor_id            as akbh_vendor_id     ,
  akbh.vendor_name          as akbh_vendor_name   ,
  akbh.game_id              as akbh_game_id       ,
  akbh.game_name            as akbh_game_name     ,
  akbh.game_category        as akbh_game_category ,
  akbh.game_type            as akbh_game_type     ,
  akbh.currency             as akbh_currency      ,
  akbh.cash                 as akbh_cash          ,
  akbh.before_cash          as akbh_before_cash   ,
  akbh.after_cash           as akbh_after_cash    ,
  akbh.group_key            as akbh_group_key     ,
  akbh.detail               as akbh_detail        ,
  akbh.is_bonus             as akbh_is_bonus      ,
  akbh.is_promo             as akbh_is_promo      ,
  akbh.is_jackpot           as akbh_is_jackpot    ,
  akbh.site_username        as akbh_site_username ,
  akbh.key                  as akbh_key           ,
  akbh.ref_id               as akbh_ref_id        ,
  akbh.o_ref_id             as akbh_o_ref_id      ,
  akbh.betting_type         as akbh_betting_type  ,
  akbh.category             as akbh_category      ,
  akbh.created_at           as akbh_created_at    ,
  akbh.utc_created_at       as akbh_utc_created_at,

  m.id                     as m_id,
  m.site_id                as m_site_id,
  m.member_class_id        as m_member_class_id,  
  m.member_level_id        as m_member_level_id,    
  m.username               as m_username,  
  m.password               as m_password,  
  m.nickname               as m_nickname,
  m.mobile_phone_number    as m_mobile_phone_number,
  m.state                  as m_state,
  m.state_changed_at       as m_state_changed_at,
  m.parent_member_id       as m_parent_member_id,
  m.child_member_count     as m_child_member_count,
  m.last_signined_ip       as m_last_signined_ip,
  m.last_signined_at       as m_last_signined_at,
  m.created_at             as m_created_at,
  m.updated_at             as m_updated_at,
  m.deleted_at             as m_deleted_at,

  m_mss.id                                as m_mss_id,
  m_mss.member_id                         as m_mss_member_id,
  m_mss.can_exchange                      as m_mss_can_exchange,
  m_mss.can_first_charge_comp             as m_mss_can_first_charge_comp,
  m_mss.can_per_charge_comp               as m_mss_can_per_charge_comp,
  m_mss.manual_payment_type_for_partner   as m_mss_manual_payment_type_for_partner,
  m_mss.settlement_type                   as m_mss_settlement_type,
  m_mss.rate_casino                       as m_mss_rate_casino,
  m_mss.rate_casino_loosing               as m_mss_rate_casino_loosing,
  m_mss.rate_casino_bacara                as m_mss_rate_casino_bacara,
  m_mss.rate_casino_roulette              as m_mss_rate_casino_roulette,
  m_mss.rate_casino_dragon_tiger          as m_mss_rate_casino_dragon_tiger,
  m_mss.rate_slot                         as m_mss_rate_slot,
  m_mss.rate_slot_loosing                 as m_mss_rate_slot_loosing,
  m_mss.rate_powerball_single             as m_mss_rate_powerball_single,
  m_mss.rate_powerball_combo              as m_mss_rate_powerball_combo,
  m_mss.rate_powerladder_single           as m_mss_rate_powerladder_single,
  m_mss.rate_powerladder_combo            as m_mss_rate_powerladder_combo,
  m_mss.rate_eos_single                   as m_mss_rate_eos_single,
  m_mss.rate_eos_combo                    as m_mss_rate_eos_combo,
  m_mss.rate_bogglepowerball_single       as m_mss_rate_bogglepowerball_single,
  m_mss.rate_bogglepowerball_combo        as m_mss_rate_bogglepowerball_combo,
  m_mss.created_at                        as m_mss_created_at,
  m_mss.updated_at                        as m_mss_updated_at



FROM api_kgon_betting_history as akbh
  INNER JOIN members m
    ON m.username = akbh.site_username
  LEFT OUTER JOIN member_settlement_settings m_mss
    ON m_mss.member_id = m.id

";

pub struct Composition {}

impl std::fmt::Debug for Composition {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Composition of members").finish()
  }
}

impl Composition {
  ///
  pub fn new() -> Composition {
    Composition {}
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: String,
  ) -> Result<Option<models::BettingHistoryModel>, Error> {
    let query = format!(
      "
    {}
    WHERE
      akbh.id = $1
    ",
      BETTING_HISTORY_QUERY
    );

    match sql_query(query)
      .bind::<diesel::sql_types::Text, _>(id)
      .get_result::<models::BettingHistoryModel>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_by_member_id(
    &self,
    conn: &diesel::PgConnection,
    member_id: uuid::Uuid,
  ) -> Result<Option<models::BettingHistoryModel>, Error> {
    let query = format!(
      "
    {}
    WHERE
      m.id = $1
    ",
      BETTING_HISTORY_QUERY
    );

    match sql_query(query)
      .bind::<diesel::sql_types::Uuid, _>(member_id)
      .get_result::<models::BettingHistoryModel>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  fn get_pagination(&self, find_all: &models::FindAll) -> Result<String, Error> {
    let mut query_pagination = String::new();

    if let Some(pagination) = &find_all.pagination {
      let page = match pagination.page {
        Some(v) => {
          if v < 1 {
            1
          } else {
            v
          }
        }
        None => 1,
      };
      if let Some(page_size) = pagination.page_size {
        write!(
          &mut query_pagination,
          " OFFSET {} LIMIT {} ",
          ((page - 1) * page_size) as i64,
          page_size as i64
        )
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
    }

    Ok(query_pagination)
  }

  fn get_ordery_by(&self, find_all: &models::FindAll) -> Result<String, Error> {
    let mut query_ordery_by = String::new();

    if let Some(sorts) = &find_all.sorts {
      for s in sorts {
        if !query_ordery_by.is_empty() {
          write!(&mut query_ordery_by, " , ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }

        let (property, sort) = match s {
          bcr::pagination::Sort::ASC(property) => (property.clone(), "ASC".to_string()),
          bcr::pagination::Sort::DESC(property) => (property.clone(), "DESC".to_string()),
        };

        let property = match property.as_str() {
          "member_id" => "m.id".to_string(),
          "username" => "m.username".to_string(),
          "nickname" => "m.nickname".to_string(),
          "vendor_id" => "akbh.vendor_id".to_string(),
          "vendor_name" => "akbh.vendor_name".to_string(),
          "game_id" => "akbh.game_id".to_string(),
          "game_name" => "akbh.game_name".to_string(),
          "game_category" => "akbh.game_category".to_string(),
          "game_type" => "akbh.game_type".to_string(),
          "currency" => "akbh.currency".to_string(),
          "cash" => "akbh.cash".to_string(),
          "before_cash" => "akbh.before_cash".to_string(),
          "after_cash" => "akbh.after_cash".to_string(),
          "key" => "akbh.key".to_string(),
          "ref_id" => "akbh.ref_id".to_string(),
          "o_ref_id" => "akbh.o_ref_id".to_string(),
          "group_key" => "akbh.group_key".to_string(),
          "is_bonus" => "akbh.is_bonus".to_string(),
          "is_promo" => "akbh.is_promo".to_string(),
          "is_jackpot" => "akbh.is_jackpot".to_string(),
          "site_username" => "akbh.site_username".to_string(),
          "betting_type" => "akbh.betting_type".to_string(),
          "category" => "category.id".to_string(),
          "created_at" => "akbh.created_at".to_string(),
          "utc_created_at" => "akbh.utc_created_at".to_string(),
          _ => "".to_string(),
        };

        if !property.is_empty() {
          write!(&mut query_ordery_by, "{} {}", property, sort)
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
      }
    }

    Ok(query_ordery_by)
  }

  fn get_where(&self, find_all: &models::FindAll) -> Result<String, Error> {
    let mut query_where = String::new();

    if let Some(s) = &find_all.search {
      if let Some(sp) = &s.member_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "m.id = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = s.vendor_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.vendor_id = {}", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if !s.vendor_ids.is_empty() {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(
          &mut query_where,
          "akbh.vendor_id IN ({})",
          s.vendor_ids
            .iter()
            .map(|v| v.to_string())
            .collect::<Vec<String>>()
            .join(",")
        )
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.vendor_name {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.vendor_name = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.vendor_name_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.vendor_name like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = s.game_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_id = {}", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if !s.game_ids.is_empty() {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(
          &mut query_where,
          "akbh.game_id IN ({})",
          s.game_ids
            .iter()
            .map(|v| v.to_string())
            .collect::<Vec<String>>()
            .join(",")
        )
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.game_name {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_name = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.game_name_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_name like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.game_category {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_category = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.game_category_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_category like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.game_type {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_type = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.game_type_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.game_type like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.currency {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.currency = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.currency_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.currency like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.key {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.key = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.key_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.key like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.ref_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.ref_id = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.ref_id_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.ref_id like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.o_ref_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.o_ref_id = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.o_ref_id_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.o_ref_id like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.group_key {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.group_key = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.group_key_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.group_key like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = s.is_bonus {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.is_bonus = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = s.is_promo {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.is_promo = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = s.is_jackpot {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.is_jackpot = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.site_username {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.site_username = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.site_username_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.site_username like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.betting_type {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.betting_type = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.betting_type_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.betting_type like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.category {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.category = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.category_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "akbh.category like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
    }

    Ok(query_where)
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<i64, Error> {
    use diesel::sql_types::BigInt;

    #[derive(QueryableByName)]
    struct Count {
      #[sql_type = "BigInt"]
      count: i64,
    }

    let mut query = String::new();
    write!(&mut query, "{}", BETTING_HISTORY_COUNT_QUERY)
      .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;

    let query_where = self.get_where(find_all)?;
    if !query_where.is_empty() {
      write!(&mut query, " WHERE {}", query_where)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }

    println!("query: {}", query);

    match sql_query(query).get_result::<Count>(conn) {
      Ok(m) => Ok(m.count),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(0),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<Vec<models::BettingHistoryModel>, Error> {
    let mut query = String::new();
    write!(&mut query, "{}", BETTING_HISTORY_QUERY)
      .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;

    let query_where = self.get_where(find_all)?;
    if !query_where.is_empty() {
      write!(&mut query, " WHERE {}", query_where)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }
    let query_ordery_by = self.get_ordery_by(find_all)?;
    if !query_ordery_by.is_empty() {
      write!(&mut query, " ORDER BY {}", query_ordery_by)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }
    let query_pagination = self.get_pagination(find_all)?;
    if !query_pagination.is_empty() {
      write!(&mut query, " {} ", query_pagination)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }

    println!("query: {}", query);

    match sql_query(query).get_results::<models::BettingHistoryModel>(conn) {
      Ok(m) => Ok(m),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(vec![]),
        _ => Err(e),
      },
    }
  }
}
